﻿-- Author      : RisM
-- Create Date : 6/28/2009 3:57:34 PM

local SpeakinSpell = LibStub("AceAddon-3.0"):GetAddon("SpeakinSpell")
local L	= LibStub("AceLocale-3.0"):GetLocale("SpeakinSpell", false)

SpeakinSpell:PrintLoading("gui/gui.lua")


-------------------------------------------------------------------------------
-- FILTER EVENT LIST SEARCH FUNCTION
-------------------------------------------------------------------------------


	
function SpeakinSpell:RebuildSpellList( values, OptionsGUIState, EventTable, MatchesFilterFunc, GetDisplayNameFunc )
	local funcname = "RebuildSpellList"
	
	if not OptionsGUIState.FilterChanged then
		--self:DebugMsg(funcname,"not changed")
		return
	end
	
	--self:DebugMsg(funcname,"entry")
	OptionsGUIState.FilterChanged = false

	-- if the current selection doesn't match the filter, erase it, and we'll pick a new one
	if OptionsGUIState.SelectedEventKey and not MatchesFilterFunc( OptionsGUIState.SelectedEventKey ) then
		--self:DebugMsg(funcname,"erasing selection")
		OptionsGUIState.SelectedEventKey = nil
	end
	
	-- search NewEventsDetected for those that match the filter
	-- add or remove from the list as needed
	numAdded = 0
	for key,_ in pairs(EventTable) do
		-- if we're allowed to add more
		-- and this event matches the filter
		if (SpeakinSpellSavedData.ShowMoreThanAHundred or (numAdded < 100)) and MatchesFilterFunc(key) then
			-- add to the list of shown values
			numAdded = numAdded + 1
			if not values[key] then
				values[key] = GetDisplayNameFunc(key)
			end
			-- make sure we have a current spell selected for the options GUI 
			-- we'll use the first one we see
			if not OptionsGUIState.SelectedEventKey then
				--self:DebugMsg(funcname,"set selection to:"..key)
				OptionsGUIState.SelectedEventKey = key
			end
		else
			-- remove it from the list of shown values
			values[key] = nil
		end
	end
	
	--table.sort(values) -- appears to be self-sorting
end


-------------------------------------------------------------------------------
-- OPTIONS GUI FUNCTIONS - SHARED UTILITIES
-------------------------------------------------------------------------------


function SpeakinSpell:AddToBlizOptions(FrameObjectName, FrameDisplayName, ParentDisplayName)
	-- AddToBlizOptions(appName="SpeakinSpell", display name, parent, ObjectName)
	-- NOTE: the FrameObjectName here represents self.OptionsGUI.args[FrameObjectName]
	self.optionsFrames[FrameObjectName] = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("SpeakinSpell", FrameDisplayName, ParentDisplayName, FrameObjectName)
	
	-- for some reason the framework has a problem if we define the "reset defaults" button handler in the data before calling AddToBlizOptions
	-- so we have to add it afterwards
	self.optionsFrames[FrameObjectName].default = function() self:ResetToDefaults(FrameObjectName) end
end



function SpeakinSpell:RefreshFrame(FrameName)
	-- the only way I found to do this is to hide and then show the frame
	-- this occurs fast enough that I do not see a flicker when this happens
	-- UPDATE: saw a forum post when I was at work with the proper API call, but I already forget what it was, and this still works
	if self.optionsFrames and self.optionsFrames[FrameName] and self.optionsFrames[FrameName]:IsVisible() then
		self.optionsFrames[FrameName]:Hide()
		self.optionsFrames[FrameName]:Show()
	end
end


-------------------------------------------------------------------------------
-- OPTIONS GUI FUNCTIONS - ENTRY POINT FOR ALL
-------------------------------------------------------------------------------

function SpeakinSpell:CreateGUI()
	-- Define Options table for the GUI
	self.OptionsGUI.name = L["SpeakinSpell"]
	
	-- add SpeakinSpell.MAX.MESSAGES_PER_SPELL random speech input boxes
	self:CurrentMessagesGUI_CreateGUI()
	self:RandomSubs_CreateGUI()
	
	-- build help pages
	self:HelpPages_CreateGUI()
	
	-- add the options to the in-game interface options panel at Escape > Interface > Addons > SpeakinSpell
	-- The ordering here matters, it determines the order in the Blizzard Interface Options
	-- (or it would matter if we had sub-categories, which we currently don't)
	-- InterfaceOptions_AddCategory(self.OptionsGUI) -- done via Ace below

	-- :RegisterOptionsTable(appName, options, slashcmd, persist)
	-- AceConfig:RegisterOptionsTable("MyAddon", myOptions, {"/myslash", "/my"})
	LibStub("AceConfig-3.0"):RegisterOptionsTable("SpeakinSpell", self.OptionsGUI, nil)
	
	--SpeakinSpell:AddToBlizOptions(FrameObjectName, FrameDisplayName, ParentDisplayName)
	--frame objects are stored to self.optionsFrames[FrameObjectName]
	self.optionsFrames = {}
	self:AddToBlizOptions("General",		L["SpeakinSpell"],		nil)
	self:AddToBlizOptions("CurrentMessagesGUI",	L["Message Settings TITLE"],		L["SpeakinSpell"])
	self:AddToBlizOptions("CreateNew",		L["Create New..."],			L["SpeakinSpell"])
	self:AddToBlizOptions("RandomSubs",		L["Random Substutitions"],	L["SpeakinSpell"])
	self:AddToBlizOptions("Trade",			L["Trade"],					L["SpeakinSpell"])
	self:AddToBlizOptions("HelpRoot",		L["SpeakinSpell Help"],		L["SpeakinSpell"])
	
	-- add help pages
	self:HelpPages_AddToBlizOptions()

	-- add the minimap button
	self:MinimapButton_Create()
end




function SpeakinSpell:ShowOptions()
	-- open interface options to show SpeakinSpell
	InterfaceOptionsFrame_OpenToCategory(L["SpeakinSpell"])
end

function SpeakinSpell:ShowOptions_Toggle()
	-- open interface options to show SpeakinSpell
	-- if it's already open, close it
	local FrameName = L["General"]
	if self.optionsFrames and self.optionsFrames[FrameName] and self.optionsFrames[FrameName]:IsVisible() then
		self.optionsFrames[FrameName]:GetParent():GetParent():Hide()
	else
		InterfaceOptionsFrame_OpenToCategory(L["SpeakinSpell"])
	end
end


function SpeakinSpell:ShowHelp()
	-- open interface options to show SpeakinSpell
	InterfaceOptionsFrame_OpenToCategory(L["SpeakinSpell Help"])
end


function SpeakinSpell:ShowCreateNew()
	-- open interface options to show SpeakinSpell
	InterfaceOptionsFrame_OpenToCategory(L["Create New..."])
end


function SpeakinSpell:ShowMessageOptions()
	-- open interface options to show SpeakinSpell
	InterfaceOptionsFrame_OpenToCategory(L["Message Settings TITLE"])
end


function SpeakinSpell:ShowRandomSubsOptions()
	-- open interface options to show SpeakinSpell
	InterfaceOptionsFrame_OpenToCategory(L["Random Substutitions"])
end

